package com.bsoft.admin.service.impl;

import com.bsoft.admin.common.enums.StateType;
import com.bsoft.admin.mapper.SysMenuMapper;
import com.bsoft.admin.mapper.SysRoleMenuRsMapper;
import com.bsoft.admin.mapper.SysUserMenuRsMapper;
import com.bsoft.admin.model.SysMenu;
import com.bsoft.admin.model.SysRole;
import com.bsoft.admin.model.SysRoleMenuRs;
import com.bsoft.admin.model.SysUserMenuRs;
import com.bsoft.admin.model.reqmodel.MenuReq;
import com.bsoft.admin.model.respmodel.SysMenuList;
import com.bsoft.admin.service.SysMenuService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.beans.Transient;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SysMenuServiceImpl implements SysMenuService {

    @Resource
    private SysMenuMapper sysMenuMapper;
    @Resource
    private SysUserMenuRsMapper sysUserMenuRsMapper;
    @Resource
    private SysRoleMenuRsMapper sysRoleMenuRsMapper;

    @Override
    public List<SysMenuList> getMenu(Long userId) throws Exception {
        SysMenuList sysMenuList = new SysMenuList();
        SysMenu sysMenu = new SysMenu();
        List<SysMenuList> list = sysMenuMapper.selectMenuByUser(userId);
        List<SysMenuList> resultList  = getLevelData(list, Long.valueOf(MENU_TOP_PARENT_ID));
        resultList=resultList.stream().sorted(Comparator.comparing(SysMenu::getSort)).collect(Collectors.toList());
        return resultList;
    }

    private List<SysMenuList> getLevelData(List<SysMenuList> list, Long parentcode) {
        List<SysMenuList> resultList = new ArrayList<>();
        for (SysMenuList data : list) {
           if (data.getParentId() == parentcode){
               List<SysMenuList> childList = getLevelData(list,data.getId());
               childList=childList.stream().sorted(Comparator.comparing(SysMenu::getSort)).collect(Collectors.toList());
               data.setSysMenuList(childList);
               resultList.add(data);
           }
        }
        return resultList;
    }

    @Override
    public List<SysMenu> getAll(MenuReq.GetMenuReq menu){
        List<SysMenu> list = sysMenuMapper.selectAll();
        if(menu.getMenuName()!=null){
            list = list.stream().filter(
                    o->(o.getMenuName()!=null && o.getMenuName().toLowerCase().contains(menu.getMenuName().toLowerCase())))
                    .collect(Collectors.toList());
        }
        return list;
    }

    @Override
    public SysMenu info(MenuReq.InfoMenuReq menu){
        SysMenu sysMenu = sysMenuMapper.selectByPrimaryKey(menu.getMenuId());
        return sysMenu;
    }

    @Override
    public boolean addMenu(Long userId,MenuReq.AddMenuReq menu){
        if(menu!=null){
            SysMenu sysMenu = new SysMenu();
            sysMenu.setCreateData(new Date());
            sysMenu.setCreateUserid(userId);
            sysMenu.setState((short) StateType.ON.getValue());
            sysMenu.setMenuName(menu.getMenuName());
            sysMenu.setMenuUrl(menu.getMenuUrl());
            sysMenu.setMenuImage(menu.getMenuImage());
            sysMenu.setPageCode(menu.getPageCode());
            sysMenu.setParentId(menu.getParentId());
            sysMenuMapper.insert(sysMenu);
            return true;
        }
        return false;
    }

    @Override
    public boolean deleteMenu(MenuReq.DeleteMenuReq menu){
        if(menu!=null){
            SysMenu sysMenu = sysMenuMapper.selectByPrimaryKey(menu.getMenuId());
            if(sysMenu!=null){
                sysMenu.setState((short) StateType.OFF.getValue());
                sysMenuMapper.updateByPrimaryKey(sysMenu);
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean updateMenu(MenuReq.UpdateMenuReq menu) {
        if(menu!=null) {
            SysMenu sysMenu = sysMenuMapper.selectByPrimaryKey(menu.getMenuId());
            if(sysMenu!=null){
                sysMenu.setMenuName(menu.getMenuName());
                sysMenu.setMenuUrl(menu.getMenuUrl());
                sysMenu.setMenuImage(menu.getMenuImage());
                sysMenu.setPageCode(menu.getPageCode());
                sysMenu.setParentId(menu.getParentId());
                sysMenuMapper.updateByPrimaryKey(sysMenu);
                return true;
            }
        }
        return false;
    }

    /**
     * 查询用户菜单关系
     * @param userId 用户ID
     * @return
     */
    @Override
    public List<SysUserMenuRs> getUserMenu(Long userId){
        List<SysUserMenuRs> list = sysUserMenuRsMapper.selectByUser(userId);
        return list;
    }

    /**
     * 用户配置菜单
     * @param userId 操作人ID
     * @param menu  用户菜单关系数据
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveUserMenu(Long userId,MenuReq.SaveUserMenuReq menu){
        if(menu!=null&&menu.getMenus()!=null&&menu.getMenus().size()>0&&menu.getUserId()!=null){
            sysUserMenuRsMapper.deleteAllByUser(menu.getUserId());
            int result = 0;
            for (Long menuId:menu.getMenus()) {
                SysUserMenuRs um = new SysUserMenuRs();
                um.setCreateDate(new Date());
                um.setState((short)StateType.ON.getValue());
                um.setCreateUserid(userId);
                um.setUserId(menu.getUserId());
                um.setMenuId(menuId);
                sysUserMenuRsMapper.insert(um);
                result++;
            }
            return result>0;
        }
        return false;
    }

    /**
     * 查询角色菜单关系
     * @param roleId 角色ID
     * @return
     */
    @Override
    public List<SysRoleMenuRs> getRoleMenu(Long roleId){
        List<SysRoleMenuRs> list = sysRoleMenuRsMapper.selectByRole(roleId);
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveRoleMenu(Long userId,MenuReq.SaveRoleMenuReq menu){
        if(menu!=null&&menu.getMenus()!=null&&menu.getMenus().size()>0&&menu.getRoleId()!=null){
            sysRoleMenuRsMapper.deleteAllByRole(menu.getRoleId());
            int result = 0;
            for (Long menuId:menu.getMenus()) {
                SysRoleMenuRs rm= new SysRoleMenuRs();
                rm.setCreateDate(new Date());
                rm.setState((short)StateType.ON.getValue());
                rm.setCreateUserid(userId);
                rm.setRoleId(menu.getRoleId());
                rm.setMenuId(menuId);
                sysRoleMenuRsMapper.insert(rm);
                result++;
            }
            return result>0;
        }
        return false;
    }


}
